/******************************************************************************
*  BC727x演示程序 STC8G 版
*  (使用驱动库，基本键盘及显示，SPI不使用中断模式，不使用KEY引脚)
*
* 程序用途：BC727x LED显示及键盘接口芯片演示，适用于所有有键盘功能的BC727x芯片
* CPU: STC8G
* RTOS:	无
* Main Frequency: 20MHz
* 版权所有:	北京凌志比高科技有限公司 https://bitcode.com.cn
* 版本号: V1.0
* 初始版本：2021年4月15日
* 版本历史:
*	2021年4月	V1.0
* 功能说明：	为了通用于所有芯片型号，显示只使用DIG0-DIG3
*		    1. 在DIG1-DIG3显示一个3位10进制计数，每50ms加一。达到999后清零。
*		       如果有按键，将键号以16进制显示在DIG0。
*		    2. 按S15计数递减，S14计数递加。
*
* 使用环境：	本程序使用GCC编译器调试通过。
* 程序说明：
*	  因为显示更新周期小于100ms，每次刷新显示均可获得键盘状态，更新周期足够短已经可以满足
*     及时键盘响应的需要，故无需使用KEY连接外部中断。
*     驱动库位于 bc727x_lib/子目录中。关于驱动库的使用更多信息请查询 《BC727x数码管及
*     键盘接口芯片驱动库技术说明书》
*     程序直接操作寄存器并不依赖于任何其它库，但可和其它库共同使用。
******************************************************************************/

/********************************* 重要 ****************************************
*********      请确认 bc727x_config.h 中驱动库设置是否和项目相符          ***********
******************************************************************************/
#include "8052.h" /* 标准51寄存器定义 */
#include "bc727x_lib/bc727x.h"
#include "stc8g.h" /* STC8G单片机扩展寄存器定义 */

/*   BC727X驱动使用IO口        */
/*   P3.1 --- CS                */
/*   P3.2 --- CLK               */
/*   P3.3 --- MISO              */
/*   P3.4/P5.4 --- MOSI         */

#define CS P3_1

/* 初始化STC8G芯片特有寄存器 */
void init_chip(void)
{
    /*	M1:M0 settings:
		0:0  traditional mode
		0:1  hi current mode
		1:0  hi-impedence
		1:1  open drain  */

    /* 设置I/O口 模式，此处均设为传统51模式 */
    P0M0 = 0;
    P0M1 = 0;
    P1M0 = 0;
    P1M1 = 0;
    P2M0 = 0;
    P2M1 = 0;
    P3M0 = 0;
    P3M1 = 0;
    P4M0 = 0;
    P4M1 = 0;
    P5M0 = 0;
    P5M1 = 0;

    P_SW2  = 0x80; /* CKSEL & CLKDIV 处于XDATA地址空间，使用MOVX A,@DPTR访问前需要先设置P_SW2的bit7为1  */
    CKSEL  = 0x00; /* 使用内部高速RC (烧录时设置内部高速RC振荡频率为 20MHz) */
    CLKDIV = 10;   /* SYSCLK = MCLK/10 得到2MHz */
    P_SW2  = 0x00; /* 回复 P_SW2 */
    AUXR   = 0;    /* 设置定时器0，1，2采用12T模式以使程序和经典51保持兼容 */
}

/* 初始化IO函数 function to initialize GPIOs */
void init_io(void)
{
    CS = 1;
}

/* 初始化SPI接口函数 function to initialize SPI1 */
void init_spi(void)
{
    SPCTL  = 0xDF; /* 设置SPI口为主机模式，高位在前，时钟空闲高电平，MISO在时钟上升沿采样，32分频 */
    SPSTAT = 0xC0; /* 清除SPI状态寄存器标志位 */
}

/* 通用延时函数 */
void delay_1ms(unsigned int Ms)
{
    uint16_t i;
    uint16_t j;
    for (i = 0; i < Ms; i++)
    {
        for (j = 0; j < 156; j++)
            ; /* 此循环大约延时1ms */
    }
}

/* SPI数据发送函数 */
void spi_send(uint8_t data)
{
    SPDAT = data;
    while (!(SPSTAT & 0x80))
        ;                     /* 等待发送完成 */
    SPSTAT = 0xC0;            /* 清除标志位 */
    update_key_status(SPDAT); /* 更新键盘状态 */
}

/* CS置高电平函数 */
void set_cs_1(void)
{
    CS = 1;
}

/* CS置低电平函数 */
void set_cs_0(void)
{
    CS = 0;
}

int main(void)
{
    int     i = 0;
    int     x = 1;
    uint8_t KeyNumber;

    init_chip();
    init_io();
    init_spi();
    set_write_func(spi_send); /* 设置spi写入操作函数 */
    set_cs_high_func(set_cs_1);
    set_cs_low_func(set_cs_0);
    delay_1ms(100); /* 启动延时，确保BC727X进入工作状态。 */
    clear();
    while (1)
    {
        display_dec(i, 1, 3); /* 从DIG1开始按10进制显示i, 宽度为3位 */
        i = i + x;
        if (i == 1000)
        {
            i = 0;
        }
        else if (i == 0)
        {
            i = 999;
        }
        if (is_key_changed()) /* 查询是否有新按键 */
        {
            KeyNumber = get_key_value(); /* 获取键值 */
            display_hex(KeyNumber, 0, 1);
            switch (KeyNumber) /* 根据键值做相应处理 */
            {
            case 15:
                x = -1;
                break;
            case 14:
                x = 1;
                break;
            default:
                break;
            }
        }
        delay_1ms(50);
    }
}

/*************************** End of file ****************************/
