/******************************************************************************
*  BC727x演示程序 STC8G 版
*  (使用驱动库，组合键+长按键，SPI不使用中断模式，不使用KEY引脚)
*
* 程序用途：BC727x LED显示及键盘接口芯片演示，适用于所有带有键盘功能的BC727x芯片
* CPU: STC8G
* RTOS:	无
* Main Frequency: 20MHz
* 版权所有:	北京凌志比高科技有限公司 https://bitcode.com.cn
* 版本号: V1.0
* 初始版本：2021年4月16日
* 版本历史:
*	2021年4月	V1.0
* 功能说明：	为了通用于所有芯片型号，显示只使用DIG0-DIG3
*		    1. 在DIG1-DIG3显示一个3位10进制计数，每50ms加一。达到999后清零。
*		       如果有按键，将键号以16进制显示在DIG0。
*		    2. 按S15计数递减，S14计数递加。
*           3. 按S0停止/开始计数，长按S0计数清零
*           4. 按S1计数手动加1(只有在停止状态下能看清),按S1+S3组合键计数手动减一
*
* 使用环境：	本程序使用SDCC编译器调试通过。
* 程序说明：
*	  因为显示更新周期小于100ms，每次刷新显示均可获得键盘状态，更新周期足够短已经可以满足
*     及时键盘响应的需要，故无需使用KEY连接外部中断。
*     驱动库位于 bc727x_lib/子目录中。关于驱动库的使用更多信息请查询 《BC727x数码管及
*     键盘接口芯片驱动库技术说明书》
*     驱动库键盘api与BC6xxx键盘驱动库高度相似，可参考BC6xxx驱动库参考更多例子程序。
*     程序直接操作寄存器并不依赖于任何其它库，但可和其它库共同使用。
******************************************************************************/

/********************************* 重要 ****************************************
*********      请确认 bc727x_config.h 中驱动库设置是否和项目相符          ***********
******************************************************************************/
#include "8052.h" /* 标准51寄存器定义 */
#include "bc727x_lib/bc727x.h"
#include "stc8g.h" /* STC8G单片机扩展寄存器定义 */

/*   BC727X驱动使用IO口        */
/*   P3.1 --- CS                */
/*   P3.2 --- CLK               */
/*   P3.3 --- MISO              */
/*   P3.4/P5.4 --- MOSI         */

#define CS P3_1

/* 初始化STC8G芯片特有寄存器 */
void init_chip(void)
{
    /*	M1:M0 settings:
		0:0  traditional mode
		0:1  hi current mode
		1:0  hi-impedence
		1:1  open drain  */

    /* 设置I/O口 模式，此处均设为传统51模式 */
    P0M0 = 0;
    P0M1 = 0;
    P1M0 = 0;
    P1M1 = 0;
    P2M0 = 0;
    P2M1 = 0;
    P3M0 = 0;
    P3M1 = 0;
    P4M0 = 0;
    P4M1 = 0;
    P5M0 = 0;
    P5M1 = 0;

    P_SW2  = 0x80; /* CKSEL & CLKDIV 处于XDATA地址空间，使用MOVX A,@DPTR访问前需要先设置P_SW2的bit7为1  */
    CKSEL  = 0x00; /* 使用内部高速RC (烧录时设置内部高速RC振荡频率为 20MHz) */
    CLKDIV = 10;   /* SYSCLK = MCLK/10 得到2MHz */
    P_SW2  = 0x00; /* 回复 P_SW2 */
}

/* 初始化IO函数 function to initialize GPIOs */
void init_io(void)
{
    CS = 1;
}

/* 初始化SPI接口函数 function to initialize SPI1 */
void init_spi(void)
{
    SPCTL  = 0xDF; /* 设置SPI口为主机模式，高位在前，时钟空闲高电平，MISO在时钟上升沿采样，32分频 */
    SPSTAT = 0xC0; /* 清除SPI状态寄存器标志位 */
}

/* 通用延时函数 */
void delay_1ms(unsigned int Ms)
{
    uint16_t i;
    uint16_t j;
    for (i = 0; i < Ms; i++)
    {
        for (j = 0; j < 156; j++)
            ; /* 此循环大约延时1ms */
    }
}

/* SPI数据发送函数 */
void spi_send(uint8_t data)
{
    SPDAT = data;
    while (!(SPSTAT & 0x80))
        ;                     /* 等待发送完成 */
    SPSTAT = 0xC0;            /* 清除标志位 */
    update_key_status(SPDAT); /* 更新键盘状态 */
}

/* CS置高电平函数 */
void set_cs_1(void)
{
    CS = 1;
}

/* CS置低电平函数 */
void set_cs_0(void)
{
    CS = 0;
}

/* 定义组合键 */
const uint16_t CBKeyDefs[] = { 0x000A }; /* 只定义一个组合键，为S1+S3，其键值将为0x10 */

/* 定义长按键 */
const uint8_t LPKeyDefs[] = { 0 }; /* 只定义一个长按键，为S0,其键值将为0x20 (0x20加原键值) */

int main(void)
{
    int     i = 0;
    int     x = 1;
    uint8_t KeyNumber;
    uint8_t Stop = 0;

    set_write_func(spi_send);   /* 设置spi写入操作函数 */
    set_cs_high_func(set_cs_1); /* 设置CS控制函数 */
    set_cs_low_func(set_cs_0);
    def_combined_key(CBKeyDefs, 1);  /* 设置组合键定义 */
    def_longpress_key(LPKeyDefs, 1); /* 设置长按键定义 */
    set_longpress_count(60);         /* 设置长按键时间为3s(50ms*60) */
    init_chip();
    init_io();
    init_spi();
    delay_1ms(100); /* 启动延时，确保BC727X进入工作状态。 */
    clear();
    while (1)
    {
        display_dec(i, 1, 3); /* 从DIG1开始按10进制显示i, 宽度为3位 */
        if (!Stop)
        {
            i = i + x;
            if (i == 1000)
            {
                i = 0;
            }
            else if (i == -1)
            {
                i = 999;
            }
        }
        if (is_key_changed()) /* 查询是否有新按键 */
        {
            KeyNumber = get_key_value(); /* 获取键值 */
            display_hex(KeyNumber, 0, 1);
            switch (KeyNumber) /* 根据键值做相应处理 */
            {
            case 15:
                x = -1;
                break;
            case 14:
                x = 1;
                break;
            case 0:
                Stop = Stop ^ 0x01; /* 每次执行Stop在0和1之间翻转 */
                break;
            case 1:        /* S1键 */
                i = i + 1; /* 计数手动加一 */
                if (i == 1000)
                {
                    i = 0;
                }
                else if (i == -1)
                {
                    i = 999;
                }
                display_dec(i, 1, 3); /* 显示计数值 */
                break;
            case 0x10:     /* 组合键的键值 （S1+S3) */
                i = i - 1; /* 计数手动减一 */
                if (i == 1000)
                {
                    i = 0;
                }
                else if (i == -1)
                {
                    i = 999;
                }
                display_dec(i, 1, 3);
                break;
            case 0 + 0x20: /* S0的长按键键值(原键值+0x20) */
                Stop = 1;  /* 停止计数 */
                i    = 0;  /* 计数值手动清零 */
                display_dec(i, 1, 3);
                break;
            default:
                break;
            }
        }
        delay_1ms(50);
        long_press_tick();
    }
}

/*************************** End of file ****************************/
