/******************************************************************************
*  BC727x演示程序 STC8G 版
*  (使用驱动库，无定期刷新，SPI不使用中断模式，使用KEY引脚，组合键+长按键)
*
* 程序用途：BC727x LED显示及键盘接口芯片演示，适用于所有带有键盘功能的BC727x芯片
* CPU: STC8G
* RTOS:	无
* Main Frequency: 20MHz
* 版权所有:	北京凌志比高科技有限公司 https://bitcode.com.cn
* 版本号: V1.0
* 初始版本：2021年4月16日
* 版本历史:
*	2021年4月	V1.0
* 功能说明：	为了通用于所有芯片型号，显示只使用DIG0-DIG3
*		    1. 在0位和1位，2位和3位各显示一个十进制数值，初始值为50
*		    2. 按S4和S0，第0-1位的数字+/-
*           3. 按组合键S3-S4和S3-S0,第2-3位的数字+/-
*           4. 以上按键长按则数字连续变化
*
* 使用环境：	本程序使用SDCC编译器调试通过。
* 程序说明：
*	  无动作时不刷新显示，有按键动作时靠KEY引脚触发外部中断获取键值
*     驱动库位于 bc727x_lib/子目录中。关于驱动库的使用更多信息请查询 《BC727x数码管及
*     键盘接口芯片驱动库技术说明书》
*     驱动库键盘api与BC6xxx键盘驱动库高度相似，可参考BC6xxx驱动库观察更多例子程序。
*     程序直接操作寄存器并不依赖于任何其它库，但可和其它库共同使用。
******************************************************************************/

/********************************* 重要 ****************************************
*********      请确认 bc727x_config.h 中驱动库设置是否和项目相符          ***********
******************************************************************************/
#include "8052.h" /* 标准51寄存器定义 */
#include "bc727x_lib/bc727x.h"
#include "stc8g.h" /* STC8G单片机扩展寄存器定义 */

/*   BC727X驱动使用IO口        */
/*   P3.1 --- CS                */
/*   P3.0 --- CLK               */
/*   P3.3 --- MISO              */
/*   P3.4/P5.4 --- MOSI         */
/*   P3.2 --- KEY               */

/* 软件SPI引脚定义 基本沿用硬件SPI引脚位置，KEY所接外部中断要求上升下降延均可触发，故用INT0，
   该引脚同时是硬件SPI口的CLK，因此将CLK改用P3.0引脚 */
#define CS   P3_1
#define CLK  P3_0
#define MISO P3_3
#define MOSI P5_4 /*在8脚封装芯片上硬件SPI口MOSI口线在P5.4, 其它封装上为P3.4 */
#define KEY  P3_2 /* 外部中断引脚 */

/* 初始化STC8G芯片特有寄存器 */
void init_chip(void)
{
    /*	M1:M0 settings:
		0:0  traditional mode
		0:1  hi current mode
		1:0  hi-impedence
		1:1  open drain  */

    /* 设置I/O口 模式，此处均设为传统51模式 */
    P0M0 = 0;
    P0M1 = 0;
    P1M0 = 0;
    P1M1 = 0;
    P2M0 = 0;
    P2M1 = 0;
    P3M0 = 0;
    P3M1 = 0;
    P4M0 = 0;
    P4M1 = 0;
    P5M0 = 0;
    P5M1 = 0;

    P_SW2  = 0x80; /* CKSEL & CLKDIV 处于XDATA地址空间，使用MOVX A,@DPTR访问前需要先设置P_SW2的bit7为1  */
    CKSEL  = 0x00; /* 使用内部高速RC (烧录时设置内部高速RC振荡频率为 20MHz) */
    CLKDIV = 10;   /* SYSCLK = MCLK/10 得到2MHz */
    P_SW2  = 0x00; /* 回复 P_SW2 */
}

/* 初始化IO函数 function to initialize GPIOs */
void init_io(void)
{
    CS   = 1;
    CLK  = 1;
    MISO = 1;
    KEY  = 1;
}

/* 初始化外部中断 */
void init_exti(void)
{
    IT0 = 0; /* STC8G 单片机 IT0=0 时，INT0 触发模式为上升沿下降沿双触发 */
    EX0 = 1;
    EA  = 1;
}

/* 通用延时函数 */
void delay_1ms(unsigned int Ms)
{
    uint16_t i;
    uint16_t j;
    for (i = 0; i < Ms; i++)
    {
        for (j = 0; j < 156; j++)
            ; /* 此循环大约延时1ms (STC8G, Sysclk=2MHz时 */
    }
}

/* 延时半个SPI时钟周期 (>7.8uS) */
void delay_half_T(void)
{
    __asm__("NOP"); /* Sysclk=2MHz, 加上呼叫和返回时间，大约耗时8us */
    __asm__("NOP");
    __asm__("NOP");
    __asm__("NOP");
    __asm__("NOP");
    __asm__("NOP");
}

/* SPI数据发送函数 */
void spi_send(uint8_t data)
{
    uint8_t i;
    uint8_t InData = 0;
    delay_half_T(); /* 确保两次数据传送间至少有1/2T的时间间隔 */
    for (i = 0; i < 8; i++)
    {
        if (data & 0x80) /* 如果发送数据为‘1’ */
        {
            MOSI = 1; /* MOSI置高 */
        }
        else
        {
            MOSI = 0; /* 否则MOSI置低 */
        }
        CLK = 0;        /* SCK置低 */
        delay_half_T(); /* 延时半个SPI时钟周期 */
        CLK = 1;        /* SCK置高 */
        delay_half_T();
        InData = InData << 1; /* 从MISO读入一个数据位 */
        if (MISO)             /* 测试MISO是否为高电平 */
        {
            InData = InData | 0x01;
        }
        data = data << 1; /* 发送数据左移1位 */
    }
    update_key_status(InData); /* 将收到的键盘映射传递给驱动库 */
}

/* CS置高电平函数 */
void set_cs_1(void)
{
    CS = 1;
}

/* CS置低电平函数 */
void set_cs_0(void)
{
    CS = 0;
}

/* 禁止外部中断函数 */
void disable_EXTI(void)
{
    EX0 = 0; /* 禁止INT0中断 */
}

/* 使能外部中断函数 */
void enable_EXTI(void)
{
    EX0 = 1; /* 使能INT0中断 */
}

/* 定义组合键 */
const uint16_t CBKeyDefs[] = { 0x8001, 0x8010 }; /* 定义2个组合键，为S15-S0, S15-S4，两个组合键键值将分别为0x10和0x11 (从0x10起顺序向后排) */

/* 定义长按键 */
const uint8_t LPKeyDefs[] = { 0, 4, 0x10, 0x11 }; /* 定义4个长按键，为S0长按，S4长按，组合键S15-S0长按(键值0x10)，组合键S15-S4长按(键值0x11) */
                                                  /* 四个长按键的键值将分别为 0x20, 0x24, 0x30, 0x31 (原键值+0x20) */

int main(void)
{
    int     num1 = 50;
    int     num2 = 50;
    uint8_t KeyNumber;

    set_write_func(spi_send);   /* 设置spi写入操作函数 */
    set_cs_high_func(set_cs_1); /* 设置CS控制函数 */
    set_cs_low_func(set_cs_0);
    def_combined_key(CBKeyDefs, 2);  /* 设置组合键定义 */
    def_longpress_key(LPKeyDefs, 4); /* 设置长按键定义 */
    set_longpress_count(40);         /* 设置长按键时间为2s(Systick中断周期1ms*2000) */
    set_detect_mode(1);              /* 因为程序需要检测长按键是否持续，因此需要检测按键释放 */

    init_chip();
    init_io();
    init_exti();

    delay_1ms(100); /* 启动延时，确保BC727X进入工作状态。 */
    disable_EXTI(); /* 因EXTI中也会使用SPI，因此在主程序BC727x操作时，应该临时禁止外部中断，以避免冲突的可能 */
    clear();
    display_dec(num1, 0, 2); /* 显示初始值 */
    display_dec(num2, 2, 2); /* 显示初始值 */
    enable_EXTI();
    while (1)
    {
        if (is_key_changed()) /* 查询是否有新按键 */
        {
            KeyNumber = get_key_value(); /* 获取键值 */
            switch (KeyNumber)           /* 根据键值做相应处理 */
            {
            case 4: /* 按键4处理 */
                num1 = num1 + 1;
                if (num1 == 100)
                {
                    num1 = 0;
                }
                disable_EXTI(); /* 因EXTI中也会使用SPI，因此在主程序BC727x操作时，应该临时禁止外部中断，以避免冲突的可能 */
                display_dec(num1, 0, 2);
                enable_EXTI();
                break;
            case 0: /* 按键0处理 */
                num1 = num1 - 1;
                if (num1 == -1)
                {
                    num1 = 99;
                }
                disable_EXTI();
                display_dec(num1, 0, 2);
                enable_EXTI();
                break;
            case 0x10: /* 0x10为组合键1的键值 （S15-S0) */
                num2 = num2 - 1;
                if (num2 == -1)
                {
                    num2 = 99;
                }
                disable_EXTI();
                display_dec(num2, 2, 2);
                enable_EXTI();
                break;
            case 0x11: /* 0x11为组合键2的键值 （S15-S4) */
                num2 = num2 + 1;
                if (num2 == 100)
                {
                    num2 = 0;
                }
                disable_EXTI();
                display_dec(num2, 2, 2);
                enable_EXTI();
                break;
            case 0 + 0x20:                           /* S0的长按键键值(原键值+0x20) */
                while (get_key_value() == KeyNumber) /* 检测长按键是否一直有效，因为开启了按键释放检测，按键释放时get_key_value()的返回值会变化 */
                {
                    num1 = num1 - 1;
                    if (num1 == -1)
                    {
                        num1 = 99;
                    }
                    disable_EXTI();
                    display_dec(num1, 0, 2);
                    enable_EXTI();
                    delay_1ms(200);
                }
                break;
            case 4 + 0x20: /* S4的长按键键值(原键值+0x20) */
                while (get_key_value() == KeyNumber)
                {
                    num1 = num1 + 1;
                    if (num1 == 100)
                    {
                        num1 = 0;
                    }
                    disable_EXTI();
                    display_dec(num1, 0, 2);
                    enable_EXTI();
                    delay_1ms(200);
                }
                break;
            case 0x10 + 0x20:                        /* S15-S0组合键的长按键键值(原键值+0x20) */
                while (get_key_value() == KeyNumber) /* 检测长按键是否一直有效，因为开启了按键释放检测，组合键释放时get_key_value()的返回值会变化 */
                {
                    num2 = num2 - 1;
                    if (num2 == -1)
                    {
                        num2 = 99;
                    }
                    disable_EXTI();
                    display_dec(num2, 2, 2);
                    enable_EXTI();
                    delay_1ms(200);
                }
                break;
            case 0x11 + 0x20: /* S15-S4组合键的长按键键值(原键值+0x20) */
                while (get_key_value() == KeyNumber)
                {
                    num2 = num2 + 1;
                    if (num2 == 100)
                    {
                        num2 = 0;
                    }
                    disable_EXTI();
                    display_dec(num2, 2, 2);
                    enable_EXTI();
                    delay_1ms(200);
                }
                break;
            default:
                break;
            }
        }
        delay_1ms(50);
        long_press_tick();
    }
}

void INT0_ISR(void) __interrupt(0) /* KEY外部中断服务程序 */
{
    send_cmd(0xff, 0xff); /* 发送伪指令以获取键盘映射 */
}

/*************************** End of file ****************************/
