/******************************************************************************
*  BC727x演示程序 STM32 版
*  (使用驱动库，基本键盘及显示，SPI不使用中断模式，不使用KEY引脚)
*
* 程序用途：BC727x LED显示及键盘接口芯片演示，适用于所有有键盘功能的BC727x芯片
* CPU: STM32F103C8
* RTOS:	无
* Main Frequency: 72MHz
* 版权所有:	北京凌志比高科技有限公司 https://bitcode.com.cn
* 版本号: V2.0
* 初始版本：2021年4月7日
* 版本历史:
*	2021年4月	V1.0
*	2021年4月	V2.0 更新为使用驱动库V3.0的API, 更改格式以兼容C89标准
* 功能说明：	为了通用于所有芯片型号，显示只使用DIG0-DIG3
*		    1. 在DIG1-DIG3显示一个3位10进制计数，每50ms加一。达到999后清零。
*		       如果有按键，将键号以16进制显示在DIG0。
*		    2. 按S15计数递减，S14计数递加。
*
* 使用环境：	本程序使用GCC编译器调试通过。
* 程序说明：
*	  因为显示更新周期小于100ms，每次刷新显示均可获得键盘状态，更新周期足够短已经可以满足
*     及时键盘响应的需要，故无需使用KEY连接外部中断。
*     驱动库位于 bc727x_lib/子目录中。关于驱动库的使用更多信息请查询 《BC727x数码管及
*     键盘接口芯片驱动库技术说明书》
*     程序直接操作寄存器并不依赖于任何其它库，但可和其它库共同使用。
******************************************************************************/

/********************************* 重要 ****************************************
*********      请确认 bc727x_config.h 中驱动库设置是否和项目相符          ***********
******************************************************************************/
#include "bc727x_lib/bc727x.h"
#include "stm32f10x.h"

/*   BC727X驱动使用SPI1口        */
/*   BC727X IS AT SPI1         */
/*   PA4 --- CS                */
/*   PA5 --- SCK               */
/*   PA6 --- MISO              */
/*   PA7 --- MOSI              */

/* I/O引脚定义 I/O pin definition */
#define BC727X_CS 0x00000010        /* CS at PA4 */

uint16_t DownCounter;        /* 延时计数器 */

/* 初始化RCC函数 function to initialize RCC */
void init_RCC(void)
{
    /* 调整APB2时钟频率以适合BC727X所要求速率 */
    RCC->CFGR |= RCC_CFGR_PPRE2_DIV8;        /* APB2 clock = HCLK/8 (9MHz) */

    /* 使能 AFIO, IOPA, SPI1 */
    RCC->APB2ENR |= RCC_APB2ENR_AFIOEN | RCC_APB2ENR_IOPAEN | RCC_APB2ENR_SPI1EN;
}

/* 初始化GPIO函数 function to initialize GPIOs */
void init_GPIO(void)
{
    /* CRL/CRH mostly used values:
     4 -- float input (default)
     3 -- push-pull 50MHz
     2 -- push-pull 2MHz
     1 -- push-pull 10MHz
     B -- alternate push-pull 50MHz
     A -- alternate push-pull 2MHz
     9 -- alternate push-pull 10MHz
     8 -- input with pull-up/pull-down
     0 -- analog input
    */

    GPIOA->CRL = 0xA8A24444;        /* CLK, MOSI设为外设输出，CS为GPIO，MISO为上拉/下拉输入 */
    GPIOA->ODR = 0x00000050;        /* 设MISO(PA6)为上拉，CS(PA4)设为高电平 */
}

/* 初始化SPI接口函数 function to initialize SPI1 */
void init_SPI1(void)
{
    /* SPI1配置为16位，pclk/256速率, 主机模式，clk空闲高电平，数据在第二个时钟沿采样 */
    /* 最终SPI1时钟频率为35.156KHz */
    SPI1->CR1 = SPI_CR1_DFF | SPI_CR1_SSM | SPI_CR1_SSI | SPI_CR1_SPE | SPI_CR1_BR
        | SPI_CR1_MSTR | SPI_CR1_CPOL | SPI_CR1_CPHA;
}

/* 初始化系统定时器 */
void init_Timer(void)
{
    SysTick_Config(SystemCoreClock / 1000);        /* 设置系统滴答定时器周期为1ms */
}

/* 通用延时函数 */
void delay_1ms(unsigned int Ms)
{
    DownCounter = Ms + 1;
    while (DownCounter != 0)        /* 每次Systick中断，DownCounter会减一 */
        ;
}

/* SPI数据发送函数 */
void spi_send(uint16_t data)
{
    SPI1->DR   = data;
    while ((SPI1->SR & SPI_SR_RXNE) == 0)
        ;                               /* 等待发送完成 wait for transmit to finish */
    update_key_status(SPI1->DR);        /* 更新键盘状态 */
}

/* CS置低函数 */
void cs_set_0(void)
{
	GPIOA->BRR = BC727X_CS;
}

/* CS置高函数 */
void cs_set_1(void)
{
	GPIOA->BSRR = BC727X_CS;
}

int main(void)
{
    int     i = 0;
    int     x = 1;
    uint8_t KeyNumber;

    init_RCC();
    init_GPIO();
    init_SPI1();
    init_Timer();
    set_write_func(spi_send);        /* 设置spi写入操作函数 */
    set_cs_high_func(cs_set_1);		 /* 设置CS控制函数 */
    set_cs_low_func(cs_set_0);
    delay_1ms(100);                  /* 启动延时，确保BC727X进入工作状态。 */
    clear();
    while (1)
    {
        display_dec(i, 1, 3);        /* 从DIG1开始按10进制显示i, 宽度为3位 */
        i = i + x;
        if (i == 1000)
        {
            i = 0;
        }
        else if (i == 0)
        {
            i = 999;
        }
        if (is_key_changed())        /* 查询是否有新按键 */
        {
            KeyNumber = get_key_value();        /* 获取键值 */
            display_hex(KeyNumber, 0, 1);
            switch (KeyNumber)        /* 根据键值做相应处理 */
            {
            case 15:
                x = -1;
                break;
            case 14:
                x = 1;
                break;
            default:
                break;
            }
        }
        delay_1ms(50);
    }
}

void SysTick_Handler(void)        /* 系统滴答定时器中断服务程序 */
{
    if (DownCounter != 0)        /* 如果计数器不为零，则减一	(通用延时用) */
    {
        --DownCounter;
    }
}

/*************************** End of file ****************************/
