/******************************************************************************
*  BC727x演示程序 STM32 版
*  (使用驱动库，无定期刷新，SPI不使用中断模式，使用KEY引脚，组合键+长按键)
*
* 程序用途：BC727x LED显示及键盘接口芯片演示，适用于所有带有键盘功能的BC727x芯片
* CPU: STM32F103C8
* RTOS:	无
* Main Frequency: 72MHz
* 版权所有:	北京凌志比高科技有限公司 https://bitcode.com.cn
* 版本号: V2.0
* 初始版本：2021年4月7日
* 版本历史:
*	2021年4月	V1.0
*	2021年4月	V2.0 更新为使用驱动库V3.0的API, 更改格式以兼容C89标准
* 功能说明：	为了通用于所有芯片型号，显示只使用DIG0-DIG3
*		    1. 在0位和1位，2位和3位各显示一个十进制数值，初始值为50
*		    2. 按S4和S0，第0-1位的数字+/-
*           3. 按组合键S3-S4和S3-S0,第2-3位的数字+/-
*           4. 以上按键长按则数字连续变化
*
* 使用环境：	本程序使用GCC编译器调试通过。
* 程序说明：
*	  无动作时不刷新显示，有按键动作时靠KEY引脚触发外部中断获取键值
*     驱动库位于 bc727x_lib/子目录中。关于驱动库的使用更多信息请查询 《BC727x数码管及
*     键盘接口芯片驱动库技术说明书》
*     驱动库键盘api与BC6xxx键盘驱动库高度相似，可参考BC6xxx驱动库观察更多例子程序。
*     程序直接操作寄存器并不依赖于任何其它库，但可和其它库共同使用。
******************************************************************************/

/********************************* 重要 ****************************************
*********      请确认 bc727x_config.h 中驱动库设置是否和项目相符          ***********
******************************************************************************/
#include "bc727x_lib/bc727x.h"
#include "stm32f10x.h"

/*   BC727X驱动使用SPI1口        */
/*   BC727X IS AT SPI1         */
/*   PA4 --- CS                */
/*   PA5 --- SCK               */
/*   PA6 --- MISO              */
/*   PA7 --- MOSI              */
/*   PA3 --- KEY               */

/* I/O引脚定义 I/O pin definition */
#define BC727X_CS 0x00000010        /* CS at PA4 */

uint16_t DownCounter;        /* 延时计数器 */

/* 初始化RCC函数 function to initialize RCC */
void init_RCC(void)
{
    /* 调整APB2时钟频率以适合BC727X所要求速率 */
    RCC->CFGR |= RCC_CFGR_PPRE2_DIV8;        /* APB2 clock = HCLK/8 (9MHz) */

    /* 使能 AFIO, IOPA, SPI1 */
    RCC->APB2ENR |= RCC_APB2ENR_AFIOEN | RCC_APB2ENR_IOPAEN | RCC_APB2ENR_SPI1EN;
}

/* 初始化GPIO函数 function to initialize GPIOs */
void init_GPIO(void)
{
    /* CRL/CRH most used values:
     4 -- float input (default)
     3 -- push-pull 50MHz
     2 -- push-pull 2MHz
     1 -- push-pull 10MHz
     B -- alternate push-pull 50MHz
     A -- alternate push-pull 2MHz
     9 -- alternate push-pull 10MHz
     8 -- input with pull-up/pull-down
     0 -- analog input
    */

    GPIOA->CRL = 0xA8A24444;        /* KEY, CLK, MOSI设为外设输出，CS为GPIO，MISO为上拉/下拉输入 */
    GPIOA->ODR = 0x00000050;        /* 设MISO(PA6)为上拉，CS(PA4)设为高电平 */
}

/* 初始化EXIT function to initialize EXTI */
void init_EXTI(void)
{
    AFIO->EXTICR[0] = AFIO_EXTICR1_EXTI3_PA;        /* 3号中断线使用 PA 口 EXTI line3 use Port A */
    EXTI->RTSR      = EXTI_RTSR_TR3;                /* line3上升沿中断使能 rising edge on line3 */
    EXTI->FTSR      = EXTI_FTSR_TR3;                /* line3下降沿中断使能 falling edge on line3 */
    EXTI->IMR |= EXTI_IMR_MR3;                      /* 使能line3中断 enable line3 interrupt */
}

/* 初始化NVIC函数 function to initialize NVIC */
void init_NVIC(void)
{
    NVIC_EnableIRQ(EXTI3_IRQn);        /* 使能EXTI3中断 enable EXTI3 interrupt */
}

/* 初始化SPI接口函数 function to initialize SPI1 */
void init_SPI1(void)
{
    /* SPI1配置为16位，pclk/256速率, 主机模式，clk空闲高电平，数据在第二个时钟沿采样 */
    /* 最终SPI1时钟频率为35.156KHz */
    SPI1->CR1 = SPI_CR1_DFF | SPI_CR1_SSM | SPI_CR1_SSI | SPI_CR1_SPE | SPI_CR1_BR
        | SPI_CR1_MSTR | SPI_CR1_CPOL | SPI_CR1_CPHA;
}

/* 初始化系统定时器 */
void init_Timer(void)
{
    SysTick_Config(SystemCoreClock / 1000);        /* 设置系统滴答定时器周期为1ms */
}

/* 通用延时函数 */
void delay_1ms(unsigned int Ms)
{
    DownCounter = Ms + 1;
    while (DownCounter != 0)        /* 每次Systick中断，DownCounter会减一 */
        ;
}

/* 禁止外部中断函数 */
void disable_EXTI(void)
{
    EXTI->IMR &= ~EXTI_IMR_MR3;        /* 禁止line3中断 disable line3 interrupt */
}

/* 使能外部中断函数 */
void enable_EXTI(void)
{
    EXTI->IMR |= EXTI_IMR_MR3;        /* 使能line3中断 enable line3 interrupt */
}

/* SPI数据发送函数 */
void spi_send(uint16_t data)
{
    SPI1->DR   = data;
    while ((SPI1->SR & SPI_SR_RXNE) == 0)
        ;                               /* 等待发送完成 wait for transmit to finish */
    update_key_status(SPI1->DR);        /* 更新键盘状态 */
}

/* CS置低函数 */
void cs_set_0(void)
{
	GPIOA->BRR = BC727X_CS;
}

/* CS置高函数 */
void cs_set_1(void)
{
	GPIOA->BSRR = BC727X_CS;
}

/* 定义组合键 */
const uint16_t CBKeyDefs[] = { 0x8001, 0x8010 };        /* 定义2个组合键，为S15-S0, S15-S4，两个组合键键值将分别为0x10和0x11 (从0x10起顺序向后排) */

/* 定义长按键 */
const uint8_t LPKeyDefs[] = { 0, 4, 0x10, 0x11 };        /* 定义4个长按键，为S0长按，S4长按，组合键S15-S0长按(键值0x10)，组合键S15-S4长按(键值0x11) */
                                                         /* 四个长按键的键值将分别为 0x20, 0x24, 0x30, 0x31 (原键值+0x20) */

int main(void)
{
    int     num1 = 50;
    int     num2 = 50;
    uint8_t KeyNumber;

    set_write_func(spi_send);               /* 设置spi写入操作函数 */
    set_cs_high_func(cs_set_1);		        /* 设置CS控制函数 */
    set_cs_low_func(cs_set_0);
    def_combined_key(CBKeyDefs, 2);         /* 设置组合键定义 */
    def_longpress_key(LPKeyDefs, 4);        /* 设置长按键定义 */
    set_longpress_count(2000);              /* 设置长按键时间为2s(Systick中断周期1ms*2000) */
    set_detect_mode(1);                     /* 因为程序需要检测长按键是否持续，因此需要检测按键释放 */
    init_RCC();
    init_GPIO();
    init_EXTI();
    init_SPI1();
    init_NVIC();
    init_Timer();
    delay_1ms(100);                   /* 启动延时，确保BC727X进入工作状态。 */
    disable_EXTI();                   /* 因EXTI中也会使用SPI，因此在主程序BC727x操作时，应该临时禁止外部中断，以避免冲突的可能 */
	clear();
    display_dec(num1, 0, 2);          /* 显示初始值 */
    display_dec(num2, 2, 2);          /* 显示初始值 */
    enable_EXTI();
    while (1)
    {
        if (is_key_changed())        /* 查询是否有新按键 */
        {
            KeyNumber = get_key_value();        /* 获取键值 */
            switch (KeyNumber)                  /* 根据键值做相应处理 */
            {
            case 4:        /* 按键4处理 */
                num1 = num1 + 1;
                if (num1 == 100)
                {
                    num1 = 0;
                }
                disable_EXTI();        /* 因EXTI中也会使用SPI，因此在主程序BC727x操作时，应该临时禁止外部中断，以避免冲突的可能 */
                display_dec(num1, 0, 2);
                enable_EXTI();
                break;
            case 0:        /* 按键0处理 */
                num1 = num1 - 1;
                if (num1 == -1)
                {
                    num1 = 99;
                }
                disable_EXTI();
                display_dec(num1, 0, 2);
                enable_EXTI();
                break;
            case 0x10:        /* 0x10为组合键1的键值 （S15-S0) */
                num2 = num2 - 1;
                if (num2 == -1)
                {
                    num2 = 99;
                }
                disable_EXTI();
                display_dec(num2, 2, 2);
                enable_EXTI();
                break;
            case 0x11:        /* 0x11为组合键2的键值 （S15-S4) */
                num2 = num2 + 1;
                if (num2 == 100)
                {
                    num2 = 0;
                }
                disable_EXTI();
                display_dec(num2, 2, 2);
                enable_EXTI();
                break;
            case 0 + 0x20:                                  /* S0的长按键键值(原键值+0x20) */
                while (get_key_value() == KeyNumber)        /* 检测长按键是否一直有效，因为开启了按键释放检测，按键释放时get_key_value()的返回值会变化 */
                {
                    num1 = num1 - 1;
                    if (num1 == -1)
                    {
                        num1 = 99;
                    }
                    disable_EXTI();
                    display_dec(num1, 0, 2);
                    enable_EXTI();
                    delay_1ms(200);
                }
                break;
            case 4 + 0x20:        /* S4的长按键键值(原键值+0x20) */
                while (get_key_value() == KeyNumber)
                {
                    num1 = num1 + 1;
                    if (num1 == 100)
                    {
                        num1 = 0;
                    }
                    disable_EXTI();
                    display_dec(num1, 0, 2);
                    enable_EXTI();
                    delay_1ms(200);
                }
                break;
            case 0x10 + 0x20:                               /* S15-S0组合键的长按键键值(原键值+0x20) */
                while (get_key_value() == KeyNumber)        /* 检测长按键是否一直有效，因为开启了按键释放检测，组合键释放时get_key_value()的返回值会变化 */
                {
                    num2 = num2 - 1;
                    if (num2 == -1)
                    {
                        num2 = 99;
                    }
                    disable_EXTI();
                    display_dec(num2, 2, 2);
                    enable_EXTI();
                    delay_1ms(200);
                }
                break;
            case 0x11 + 0x20:        /* S15-S4组合键的长按键键值(原键值+0x20) */
                while (get_key_value() == KeyNumber)
                {
                    num2 = num2 + 1;
                    if (num2 == 100)
                    {
                        num2 = 0;
                    }
                    disable_EXTI();
                    display_dec(num2, 2, 2);
                    enable_EXTI();
                    delay_1ms(200);
                }
                break;
            default:
                break;
            }
        }
    }
}

void SysTick_Handler(void)       /* 系统滴答定时器中断服务程序 */
{
    if (DownCounter != 0)        /* 如果计数器不为零，则减一	(通用延时用) */
    {
        --DownCounter;
    }
    long_press_tick();        /* 长按键计时 */
}

void EXTI3_IRQHandler(void)         /* KEY外部中断服务程序 */
{
    EXTI->PR |= EXTI_PR_PR3;        /* 清除中断标志 clear line4 pending bit */
    send_cmd(0xff, 0xff);           /* 发送伪指令以获取键盘映射 */
}

/*************************** End of file ****************************/
